@@ -1,5 +1,8 @@
-/*
- * Copyright (C) 2005 Richard Hughes <richard@hughsie.com>
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
+ * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ *
+ * Licensed under the GNU General Public License Version 2
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,9 +17,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Authors: Richard Hughes <richard@hughsie.com>
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -39,6 +39,7 @@
 #include <gtk/gtk.h>
 
 #include "gpm-common.h"
+#include "gpm-debug.h"
 #include "gpm-stock-icons.h"
 #include "gpm-brightness.h"
 #include "gpm-hal.h"
@@ -101,7 +102,7 @@ gpm_brightness_constructor (GType                  type,
 	GpmBrightness      *brightness;
 	GpmBrightnessClass *klass;
 	klass = GPM_BRIGHTNESS_CLASS (g_type_class_peek (GPM_TYPE_BRIGHTNESS));
-	brightness = GPM_BRIGHTNESS (G_OBJECT_CLASS (gpm_brightness_parent_class)->constructor 
+	brightness = GPM_BRIGHTNESS (G_OBJECT_CLASS (gpm_brightness_parent_class)->constructor
 			      	     (type, n_construct_properties, construct_properties));
 	return G_OBJECT (brightness);
 }
@@ -130,7 +131,7 @@ gpm_brightness_init (GpmBrightness *brightness)
 	gpm_hal_find_device_capability ("laptop_panel", &names);
 	if (names == NULL || names[0] == NULL) {
 		brightness->priv->has_hardware = FALSE;
-		g_debug ("No devices of capability laptop_panel");
+		gpm_debug ("No devices of capability laptop_panel");
 		return;
 	}
 
@@ -153,8 +154,8 @@ gpm_brightness_init (GpmBrightness *brightness)
 	/* this changes under our feet */
 	gpm_brightness_level_update_hw (brightness);
 
-	g_debug ("gpm_brightness_init: (%i of %i)", brightness->priv->current_hw,
-		 brightness->priv->levels - 1);
+	gpm_debug ("Starting: (%i of %i)", brightness->priv->current_hw,
+		   brightness->priv->levels - 1);
 }
 
 static void
@@ -193,7 +194,7 @@ gpm_brightness_level_update_hw (GpmBrightness *brightness)
 				G_TYPE_INVALID,
 				G_TYPE_UINT, &brightness_hw, G_TYPE_INVALID)) {
 		if (error) {
-			g_warning ("gpm_brightness_level_update_hw: %s", error->message);
+			gpm_warning ("%s", error->message);
 			g_error_free (error);
 		}
 		retval = FALSE;
@@ -212,22 +213,21 @@ gpm_brightness_level_set_hw (GpmBrightness *brightness,
 	gint     ret;
 	gboolean retval;
 
-	if (brightness_level_hw < 0 || 
+	if (brightness_level_hw < 0 ||
 	    brightness_level_hw > brightness->priv->levels - 1) {
-		g_warning ("gpm_brightness_level_set_hw: set outside range (%i of %i)",
-			   brightness_level_hw, brightness->priv->levels - 1);
+		gpm_warning ("set outside range (%i of %i)",
+			     brightness_level_hw, brightness->priv->levels - 1);
 		return FALSE;
 	}
 
-	g_debug ("gpm_brightness_level_set_hw: Setting %i of %i",
-		 brightness_level_hw, brightness->priv->levels - 1);
+	gpm_debug ("Setting %i of %i", brightness_level_hw, brightness->priv->levels - 1);
 
 	retval = TRUE;
 	if (!dbus_g_proxy_call (brightness->priv->proxy, "SetBrightness", &error,
 				G_TYPE_INT, brightness_level_hw, G_TYPE_INVALID,
 				G_TYPE_UINT, &ret, G_TYPE_INVALID)) {
 		if (error) {
-			g_warning ("gpm_brightness_level_set_hw: %s", error->message);
+			gpm_warning ("%s", error->message);
 			g_error_free (error);
 		}
 		retval = FALSE;
