@@ -1,6 +1,9 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ *
+ * Licensed under the GNU General Public License Version 2
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,11 +18,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Authors:
- *          William Jon McCann <mccann@jhu.edu>
- *          Richard Hughes <richard@hughsie.com>
- *
  */
 
 #include "config.h"
@@ -42,6 +40,7 @@
 #include "gpm-prefs.h"
 #include "gpm-hal.h"
 #include "gpm-marshal.h"
+#include "gpm-debug.h"
 
 #include "gpm-hal-monitor.h"
 
@@ -54,7 +53,8 @@ static void     gpm_hal_monitor_finalize   (GObject            *object);
 struct GpmHalMonitorPrivate
 {
 	DBusGConnection *connection;
-	DBusGProxy	*proxy;
+	DBusGProxy	*proxy_hal;
+	DBusGProxy	*proxy_dbus;
 
 	GHashTable      *devices;
 
@@ -68,6 +68,8 @@ enum {
 	BATTERY_PROPERTY_MODIFIED,
 	BATTERY_ADDED,
 	BATTERY_REMOVED,
+	HAL_CONNECTED,
+	HAL_DISCONNECTED,
 	LAST_SIGNAL
 };
 
@@ -169,14 +171,32 @@ gpm_hal_monitor_class_init (GpmHalMonitorClass *klass)
 			      NULL,
 			      NULL,
 			      g_cclosure_marshal_VOID__STRING,
-			      G_TYPE_NONE,
-			      1, G_TYPE_STRING);
+			      G_TYPE_NONE, 1, G_TYPE_STRING);
+	signals [HAL_CONNECTED] =
+		g_signal_new ("hal-connected",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GpmHalMonitorClass, hal_connected),
+			      NULL,
+			      NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
+	signals [HAL_DISCONNECTED] =
+		g_signal_new ("hal-disconnected",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GpmHalMonitorClass, hal_disconnected),
+			      NULL,
+			      NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
 }
 
 static void
 monitor_change_on_ac (GpmHalMonitor *monitor,
 		      gboolean	     on_ac)
 {
+	gpm_debug ("emitting ac-power-changed : %i", on_ac);
 	g_signal_emit (monitor, signals [AC_POWER_CHANGED], 0, on_ac);
 }
 
@@ -196,11 +216,7 @@ watch_device_property_modified (DBusGProxy    *proxy,
 				gboolean       is_removed,
 				GpmHalMonitor *monitor)
 {
-
-#if 1
-	g_debug ("watch_device_property_modified: udi=%s, key=%s, added=%i, removed=%i",
-		 udi, key, is_added, is_removed);
-#endif
+	gpm_debug ("udi=%s, key=%s, added=%i, removed=%i", udi, key, is_added, is_removed);
 
 	/* only process modified entries, not added or removed keys */
 	if (is_removed || is_added) {
@@ -219,6 +235,7 @@ watch_device_property_modified (DBusGProxy    *proxy,
 	if (strncmp (key, "battery", 7) != 0)
 		return;
 
+	gpm_debug ("emitting battery-property-modified : %s, %s", udi, key);
 	g_signal_emit (monitor, signals [BATTERY_PROPERTY_MODIFIED], 0, udi, key);
 }
 
@@ -236,14 +253,14 @@ watch_device_properties_modified (DBusGProxy    *proxy,
 	guint        i;
 
 	udi = dbus_g_proxy_get_path (proxy);
-	g_debug ("property modified '%s'", udi);
+	gpm_debug ("property modified '%s'", udi);
 
 	array = NULL;
 
 	for (i = 0; i < properties->len; i++) {
 		array = g_ptr_array_index (properties, i);
 		if (array->n_values != 3) {
-			g_warning ("array->n_values invalid (!3)");
+			gpm_warning ("array->n_values invalid (!3)");
 			return;
 		}
 
@@ -258,9 +275,9 @@ watch_device_properties_modified (DBusGProxy    *proxy,
 /** Invoked when a property of a device in the Global Device List is
  *  changed, and we have we have subscribed to changes for that device.
  *
- *  @param	udi			Univerisal Device Id
+ *  @param	udi		Univerisal Device Id
  *  @param	name		Name of condition
- *  @param	details	D-BUS message with parameters
+ *  @param	details		D-BUS message with parameters
  */
 static void
 watch_device_condition (DBusGProxy    *proxy,
@@ -276,8 +293,7 @@ watch_device_condition (DBusGProxy    *proxy,
 	g_assert (name);
 	g_assert (details);
 
-	g_debug ("watch_device_condition: udi=%s, name=%s, details=%s",
-		 udi, name, details);
+	gpm_debug ("udi=%s, name=%s, details=%s", udi, name, details);
 
 	if (strcmp (name, "ButtonPressed") == 0) {
 		char	 *type = NULL;
@@ -286,11 +302,16 @@ watch_device_condition (DBusGProxy    *proxy,
 		gpm_hal_device_get_string (udi, "button.type", &type);
 
 		if (!type) {
-			g_warning ("You must have a button type for %s!", udi);
+			gpm_warning ("You must have a button type for %s!", udi);
 			return;
 		}
 
-		g_debug ("ButtonPressed : %s", type);
+		if (strcmp (type, "") == 0) {
+			if (details != NULL)
+				type = g_strdup (details);
+		}
+
+		gpm_debug ("ButtonPressed : %s", type);
 
 		if (strcmp (type, "power") == 0) {
 			value = TRUE;
@@ -302,15 +323,16 @@ watch_device_condition (DBusGProxy    *proxy,
 			value = TRUE;
 
 			if (!details) {
-				g_warning ("Virtual buttons must have details for %s!", udi);
+				gpm_warning ("Virtual buttons must have details for %s!", udi);
 				return;
 			}
 		} else {
-			g_warning ("Button '%s' unrecognised", type);
+			gpm_warning ("Button '%s' unrecognised", type);
 			g_free (type);
 			return;
 		}
 
+		gpm_debug ("emitting button-pressed : %s, %s (%i)", type, details, value);
 		g_signal_emit (monitor, signals [BUTTON_PRESSED], 0, type, details, value);
 
 		g_free (type);
@@ -325,7 +347,7 @@ watch_device_connect_condition (GpmHalMonitor *monitor,
 
 	proxy = g_hash_table_lookup (monitor->priv->devices, udi);
 	if (proxy == NULL) {
-		g_warning ("Device is not being watched: %s", udi);
+		gpm_warning ("Device is not being watched: %s", udi);
 		return;
 	}
 
@@ -346,7 +368,7 @@ watch_device_connect_property_modified (GpmHalMonitor *monitor,
 
 	proxy = g_hash_table_lookup (monitor->priv->devices, udi);
 	if (proxy == NULL) {
-		g_warning ("Device is not being watched: %s", udi);
+		gpm_warning ("Device is not being watched: %s", udi);
 		return;
 	}
 
@@ -370,7 +392,7 @@ watch_device_disconnect_condition (GpmHalMonitor *monitor,
 
 	proxy = g_hash_table_lookup (monitor->priv->devices, udi);
 	if (proxy == NULL) {
-		g_warning ("Device is not being watched: %s", udi);
+		gpm_warning ("Device is not being watched: %s", udi);
 		return;
 	}
 
@@ -387,7 +409,7 @@ watch_device_disconnect_property_modified (GpmHalMonitor *monitor,
 
 	proxy = g_hash_table_lookup (monitor->priv->devices, udi);
 	if (proxy == NULL) {
-		g_warning ("Device is not being watched: %s", udi);
+		gpm_warning ("Device is not being watched: %s", udi);
 		return;
 	}
 
@@ -402,15 +424,15 @@ watch_device_add (GpmHalMonitor *monitor,
 	DBusGProxy *proxy;
 	GError     *error;
 
-	g_debug ("Adding new device to watch: %s", udi);
+	gpm_debug ("Adding new device to watch: %s", udi);
 
 	proxy = g_hash_table_lookup (monitor->priv->devices, udi);
 	if (proxy != NULL) {
-		g_warning ("Device is already being watched: %s", udi);
+		gpm_warning ("Device is already being watched: %s", udi);
 		return FALSE;
 	}
 
-	g_debug ("Creating proxy for: %s", udi);
+	gpm_debug ("Creating proxy for: %s", udi);
 	error = NULL;
 	proxy = dbus_g_proxy_new_for_name_owner (monitor->priv->connection,
 						 HAL_DBUS_SERVICE,
@@ -418,7 +440,7 @@ watch_device_add (GpmHalMonitor *monitor,
 						 HAL_DBUS_INTERFACE_DEVICE,
 						 &error);
 	if (proxy == NULL) {
-		g_warning ("Could not create proxy for UDI: %s: %s", udi, error->message);
+		gpm_warning ("Could not create proxy for UDI: %s: %s", udi, error->message);
 		g_error_free (error);
 		return FALSE;
 	}
@@ -438,7 +460,7 @@ watch_device_remove (GpmHalMonitor *monitor,
 
 	proxy = g_hash_table_lookup (monitor->priv->devices, udi);
 	if (proxy == NULL) {
-		g_warning ("Device is not being watched");
+		gpm_warning ("Device is not being watched: %s", udi);
 		return FALSE;
 	}
 
@@ -454,7 +476,7 @@ watch_add_battery (GpmHalMonitor *monitor,
 	watch_device_add (monitor, udi);
 	watch_device_connect_property_modified (monitor, udi);
 
-	g_debug ("Emitting battery-added signal for: %s", udi);
+	gpm_debug ("emitting battery-added : %s", udi);
 	g_signal_emit (monitor, signals [BATTERY_ADDED], 0, udi);
 }
 
@@ -466,6 +488,7 @@ watch_remove_battery (GpmHalMonitor *monitor,
 	watch_device_disconnect_property_modified (monitor, udi);
 	watch_device_remove (monitor, udi);
 
+	gpm_debug ("emitting battery-removed : %s", udi);
 	g_signal_emit (monitor, signals [BATTERY_REMOVED], 0, udi);
 }
 
@@ -490,9 +513,9 @@ hal_device_removed (DBusGProxy    *proxy,
 		    const char    *udi,
 		    GpmHalMonitor *monitor)
 {
-	g_debug ("hal_device_removed: udi=%s", udi);
+	gpm_debug ("udi=%s", udi);
 
-	/* these may not all be batteries but oh well */
+	/* FIXME: these may not all be batteries */
 	watch_remove_battery (monitor, udi);
 }
 
@@ -502,8 +525,7 @@ hal_new_capability (DBusGProxy    *proxy,
 		    const char    *capability,
 		    GpmHalMonitor *monitor)
 {
-	g_debug ("hal_device_new_capability: udi=%s, capability=%s",
-		 udi, capability);
+	gpm_debug ("udi=%s, capability=%s", udi, capability);
 
 	if (strcmp (capability, "battery") == 0) {
 		watch_add_battery (monitor, udi);
@@ -513,27 +535,53 @@ hal_new_capability (DBusGProxy    *proxy,
 static void
 hal_disconnect_signals (GpmHalMonitor *monitor)
 {
-	dbus_g_proxy_disconnect_signal (monitor->priv->proxy, "DeviceRemoved",
-					G_CALLBACK (hal_device_removed), monitor);
-	dbus_g_proxy_disconnect_signal (monitor->priv->proxy, "NewCapability",
-					G_CALLBACK (hal_new_capability), monitor);
+	gpm_debug ("Disconnecting signals from HAL");
+
+	if (monitor->priv->proxy_hal) {
+		/* it looks like we leak memory, but I'm pretty sure dbus
+		   cleans up the proxy for us */
+		dbus_g_proxy_disconnect_signal (monitor->priv->proxy_hal, "DeviceRemoved",
+						G_CALLBACK (hal_device_removed), monitor);
+		dbus_g_proxy_disconnect_signal (monitor->priv->proxy_hal, "NewCapability",
+						G_CALLBACK (hal_new_capability), monitor);
+		g_object_unref (monitor->priv->proxy_hal);
+		monitor->priv->proxy_hal = NULL;
+	}
+	/* we are now not using HAL */
+	monitor->priv->enabled = FALSE;
+
+	gpm_debug ("emitting hal-disconnected");
+	g_signal_emit (monitor, signals [HAL_DISCONNECTED], 0);
 }
 
 static void
 hal_connect_signals (GpmHalMonitor *monitor)
 {
-	dbus_g_proxy_add_signal (monitor->priv->proxy, "DeviceRemoved",
+	GError *error;
+
+	gpm_debug ("Connecting signals to HAL");
+	monitor->priv->proxy_hal = dbus_g_proxy_new_for_name_owner (monitor->priv->connection,
+								    HAL_DBUS_SERVICE,
+								    HAL_DBUS_PATH_MANAGER,
+								    HAL_DBUS_INTERFACE_MANAGER,
+								    &error);
+
+	dbus_g_proxy_add_signal (monitor->priv->proxy_hal, "DeviceRemoved",
 				 G_TYPE_STRING, G_TYPE_INVALID);
-	dbus_g_proxy_connect_signal (monitor->priv->proxy, "DeviceRemoved",
+	dbus_g_proxy_connect_signal (monitor->priv->proxy_hal, "DeviceRemoved",
 				     G_CALLBACK (hal_device_removed), monitor, NULL);
 
 	dbus_g_object_register_marshaller (gpm_marshal_VOID__STRING_STRING,
 					   G_TYPE_NONE, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);
-	dbus_g_proxy_add_signal (monitor->priv->proxy, "NewCapability",
+	dbus_g_proxy_add_signal (monitor->priv->proxy_hal, "NewCapability",
 				 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);
-	dbus_g_proxy_connect_signal (monitor->priv->proxy, "NewCapability",
+	dbus_g_proxy_connect_signal (monitor->priv->proxy_hal, "NewCapability",
 				     G_CALLBACK (hal_new_capability), monitor, NULL);
+	/* we are now using HAL */
+	monitor->priv->enabled = TRUE;
 
+	gpm_debug ("emitting hal-connected");
+	g_signal_emit (monitor, signals [HAL_CONNECTED], 0);
 }
 
 static gboolean
@@ -545,7 +593,7 @@ coldplug_acadapter (GpmHalMonitor *monitor)
 	/* devices of type ac_adapter */
 	gpm_hal_find_device_capability ("ac_adapter", &device_names);
 	if (! device_names) {
-		g_debug ("Couldn't obtain list of ac_adapters");
+		gpm_debug ("Couldn't obtain list of ac_adapters");
 		return FALSE;
 	}
 
@@ -567,7 +615,7 @@ coldplug_buttons (GpmHalMonitor *monitor)
 	/* devices of type button */
 	gpm_hal_find_device_capability ("button", &device_names);
 	if (! device_names) {
-		g_debug ("Couldn't obtain list of buttons");
+		gpm_debug ("Couldn't obtain list of buttons");
 		return FALSE;
 	}
 
@@ -593,7 +641,7 @@ coldplug_batteries (GpmHalMonitor *monitor)
 	/* devices of type battery */
 	gpm_hal_find_device_capability ("battery", &device_names);
 	if (! device_names) {
-		g_debug ("Couldn't obtain list of batteries");
+		gpm_debug ("Couldn't obtain list of batteries");
 		return FALSE;
 	}
 
@@ -607,37 +655,109 @@ coldplug_batteries (GpmHalMonitor *monitor)
 }
 
 static void
-hal_monitor_start (GpmHalMonitor *monitor)
+coldplug_all (GpmHalMonitor *monitor)
 {
-	GError *error;
-
-	if (monitor->priv->proxy) {
-		g_warning ("Monitor already started");
-		return;
-	}
-
-	error = NULL;
-	monitor->priv->proxy = dbus_g_proxy_new_for_name_owner (monitor->priv->connection,
-								HAL_DBUS_SERVICE,
-								HAL_DBUS_PATH_MANAGER,
-								HAL_DBUS_INTERFACE_MANAGER,
-								&error);
-	hal_connect_signals (monitor);
-
 	/* sets up these devices and adds watches */
+	gpm_debug ("coldplugging all devices");
 	coldplug_batteries (monitor);
 	coldplug_acadapter (monitor);
 	coldplug_buttons (monitor);
 }
 
-static void
-hal_monitor_stop (GpmHalMonitor *monitor)
-{
-	hal_disconnect_signals (monitor);
 
-	if (monitor->priv->proxy) {
-		g_object_unref (monitor->priv->proxy);
-		monitor->priv->proxy = NULL;
+static void
+remove_batteries_in_hash (const char *udi, gpointer value, GpmHalMonitor *monitor)
+{
+	watch_remove_battery (monitor, udi);
+}
+
+static void
+gpm_hash_free_devices_cache (GpmHalMonitor *monitor)
+{
+	if (! monitor->priv->devices) {
+		return;
+	}
+	gpm_debug ("freeing cache");
+	g_hash_table_foreach (monitor->priv->devices,
+			      (GHFunc) remove_batteries_in_hash,
+			      monitor);
+	g_hash_table_destroy (monitor->priv->devices);
+	monitor->priv->devices = NULL;
+}
+
+static void
+gpm_hash_new_devices_cache (GpmHalMonitor *monitor)
+{
+	if (monitor->priv->devices) {
+		return;
+	}
+	gpm_debug ("creating cache");
+	monitor->priv->devices = g_hash_table_new_full (g_str_hash,
+							g_str_equal,
+							g_free,
+							(GDestroyNotify)g_object_unref);
+}
+
+static void
+hal_name_owner_changed (DBusGProxy *proxy,
+			const char *name,
+			const char *prev,
+			const char *new,
+			GpmHalMonitor *monitor)
+{
+	if (strcmp (name, HAL_DBUS_SERVICE) != 0) {
+		gpm_debug ("ignoring name change: %s", name);
+		return;
+	}
+
+	gpm_debug ("prev=%s, new=%s", prev, new);
+	if (strlen (prev) != 0 && strlen (new) == 0 ) {
+		if (monitor->priv->enabled) {
+			/* We are already connected to HAL. A bug in DBUS can
+			   sometimes trigger a double n-o-c signal */
+			hal_disconnect_signals (monitor);
+
+			/* we have to rebuild the cache */
+			gpm_hash_free_devices_cache (monitor);
+			gpm_hash_new_devices_cache (monitor);
+		}
+	}
+	if (strlen (prev) == 0 && strlen (new) != 0 ) {
+		if (! monitor->priv->enabled) {
+			/* We are already connected to HAL. A bug in DBUS can
+			   sometimes trigger a double n-o-c signal */
+			hal_connect_signals (monitor);
+			coldplug_all (monitor);
+		}
+	}
+}
+
+static void
+hal_monitor_start (GpmHalMonitor *monitor)
+{
+	GError *error;
+
+	if (monitor->priv->proxy_hal) {
+		gpm_warning ("Monitor already started");
+		return;
+	}
+
+	error = NULL;
+	monitor->priv->proxy_dbus = dbus_g_proxy_new_for_name_owner (monitor->priv->connection,
+								     DBUS_SERVICE_DBUS,
+								     DBUS_PATH_DBUS,
+						 		     DBUS_INTERFACE_DBUS,
+								     &error);
+
+	dbus_g_proxy_add_signal (monitor->priv->proxy_dbus, "NameOwnerChanged",
+				 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);
+	dbus_g_proxy_connect_signal (monitor->priv->proxy_dbus, "NameOwnerChanged",
+				     G_CALLBACK (hal_name_owner_changed),
+				     monitor, NULL);
+
+	if (monitor->priv->enabled) {
+		hal_connect_signals (monitor);
+		coldplug_all (monitor);
 	}
 }
 
@@ -668,25 +788,23 @@ gpm_hal_monitor_init (GpmHalMonitor *monitor)
 	monitor->priv = GPM_HAL_MONITOR_GET_PRIVATE (monitor);
 
 	monitor->priv->enabled = gpm_hal_is_running ();
+	monitor->priv->proxy_hal = NULL;
+	monitor->priv->devices = NULL;
 
 	if (! monitor->priv->enabled) {
-		g_warning ("%s cannot connect to HAL!", NICENAME);
+		gpm_warning ("%s cannot connect to HAL!", GPM_NAME);
 	}
 
-	monitor->priv->devices = g_hash_table_new_full (g_str_hash,
-							g_str_equal,
-							g_free,
-							(GDestroyNotify)g_object_unref);
+	gpm_hash_new_devices_cache (monitor);
 
 	error = NULL;
 	monitor->priv->connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
-
 	/* FIXME: check error */
 
 	monitor->priv->has_power_management = gpm_hal_has_power_management ();
 
 	if (! monitor->priv->has_power_management) {
-		g_warning ("HAL does not have modern PowerManagement capability");
+		gpm_warning ("HAL does not have modern PowerManagement capability");
 	}
 
 	if (monitor->priv->enabled
@@ -707,10 +825,13 @@ gpm_hal_monitor_finalize (GObject *object)
 
 	g_return_if_fail (monitor->priv != NULL);
 
-	hal_monitor_stop (monitor);
+	hal_disconnect_signals (monitor);
 
-	if (monitor->priv->devices != NULL) {
-		g_hash_table_destroy (monitor->priv->devices);
+	gpm_hash_free_devices_cache (monitor);
+
+	if (monitor->priv->proxy_dbus) {
+		g_object_unref (monitor->priv->proxy_dbus);
+		monitor->priv->proxy_dbus = NULL;
 	}
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
