@@ -1,15 +1,15 @@
-/** @file	gpm-stock-icons.h
- *  @brief	Register our custom icons as GNOME stock icons
- *  @author	2002		Jorn Baayen
- *		2003,2004	Colin Walters <walters@verbum.org>
- */
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
+ * Copyright (C) 2002 Jorn Baayen
+ * Copyright (C) 2003,2004 Colin Walters <walters@verbum.org>
+ * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ *
  * Licensed under the GNU General Public License Version 2
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,8 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #ifndef __GPM_STOCK_ICONS_H
@@ -59,8 +58,12 @@ G_BEGIN_DECLS
 #define GPM_STOCK_UPS_7_OF_8	"gnome-power-ups-7-of-8"
 #define GPM_STOCK_UPS_8_OF_8	"gnome-power-ups-8-of-8"
 
-gboolean gpm_stock_icons_init (void);
-void gpm_stock_icons_shutdown (void);
+#define GNOME_DEV_BATTERY       "gnome-dev-battery"
+#define GNOME_DEV_HARDDISK      "gnome-dev-harddisk"
+#define GNOME_DEV_MEMORY        "gnome-dev-memory"
+
+gboolean gpm_stock_icons_init     (void);
+void     gpm_stock_icons_shutdown (void);
 
 G_END_DECLS
 
